<?php
namespace App\Rclone;

use Symfony\Component\Process\Process;
use Symfony\Component\Process\Exception\ProcessFailedException;

class Rclone
{
    protected string $rclone_path;
    /**
     * Global options
     * 
     * @var array<string> 
     */
    protected array $global_options = [];

    function __construct(string $rclone_path = "rclone")
    {
        $this->rclone_path = $rclone_path;
    }

    function getVersion(): string
    {
        return $this->exec('--version');
    }

    function getSize(string $path): int
    {
        $output = $this->exec('size', ['--json', $path]);
        return (int)json_decode($output)->bytes;
    }

    function copy(string $src, string $dest, string $bandwidth = null): string
    {   
        $options = array();

        $options[] = $src;
        $options[] = $dest;
        if ($bandwidth) {
            $options[] = "--bwlimit";
            $options[] = $bandwidth;
        }

        return $this->exec('copy', $options);
    }

    /**
     * @param $command Top level Rclone command
     * @param array<String> $options
     */
    protected function exec(string $command, array $options = array()) : string
    {   
        $process = new Process(
            array_merge(
                [$this->rclone_path],
                $this->global_options,
                [$command],
                $options
            )
        );
        $process->setTimeout(4*3600);
        $process->run();

        // executes after the command finishes
        if (!$process->isSuccessful()) {
            throw new ProcessFailedException($process);
        }

        return $process->getOutput();
    }
}